classdef GtThreshold < handle
    properties
        conf;

        % Reconstruction parameters from parameters.mat
        param;
    end

    properties (Constant)
        backspace = '\b';
    end

    methods
        function obj = GtThreshold(varargin)
            obj.conf = [];
            obj.conf.verbose = true;

            obj.param = load('parameters.mat');
            obj.param = obj.param.parameters;

            [obj.param.rec, rej_params] = parse_pv_pairs(obj.param.rec, varargin);
            if (~isfield(obj.param.rec, 'do_morph_recon'))
                obj.param.rec.do_morph_recon = false;
            end

            obj.conf = parse_pv_pairs(obj.conf, rej_params);
        end

        function [defaults, sample] = generateDefaults(obj, fileTable)
            samplefile  = fullfile(obj.param.acq.dir, '4_grains', 'sample.mat');
            if (exist('fileTable', 'var'))
                sample = GtSample.loadFromLockedFile(fileTable, samplefile);
            else
                sample = GtSample.loadFromFile(samplefile);
            end

            defaults = [];
            defaults.percentile = obj.param.rec.percentile2;

            defaults.phases = {};
            for phaseID = 1:length(sample.phases)
                defaults.phases{phaseID}.min_id = 1;
                defaults.phases{phaseID}.max_id = length(sample.phases{phaseID}.grains);
            end
        end

        function [varargout] = batchThreshold(obj, phaseID, firstID, lastID, percentile)
        % GTTHRESHOLD/BATCHTHRESHOLD

            global GT_DB
            gtDBConnect(GT_DB.host, GT_DB.user, GT_DB.password, GT_DB.name);
            filetable = [obj.param.acq.name '_filetable'];

            [defaults, sample] = obj.generateDefaults(filetable);

            if (exist('percentile', 'var'))
                obj.param.rec.percentile2 = percentile;

                parameters = obj.param;
                save('parameters.mat', 'parameters');
            end

            if ischar(phaseID)
                phaseID = str2double(phaseID);
            end

            % Here is for the lazy who wants to launch from CMD line OR for OAR.
            if (ischar(firstID))
                if (strcmpi(firstID, 'first'))
                    firstID = defaults.phases{phaseID}.min_id;
                else
                    firstID = str2double(firstID);
                end
            end
            if (ischar(lastID))
                if (strcmpi(lastID, 'last'))
                    lastID = defaults.phases{phaseID}.max_id;
                else
                    lastID = str2double(lastID);
                end
            end

            if ((firstID < defaults.phases{phaseID}.min_id) ...
                    || (firstID > lastID) ...
                    || (lastID > defaults.phases{phaseID}.max_id))
                error('SEGMENT:wrong_argument', ...
                      ['The range indexes are not correct: (%d, %d).\n' ...
                      'Largest bounds (%d, %d)'], ...
                      firstID, lastID, defaults.phases{phaseID}.min_id, ...
                      defaults.phases{phaseID}.max_id);
            end

            phaseDir = fullfile(obj.param.acq.dir, '4_grains', ...
                                sprintf('phase_%02d', phaseID));

            gauge = GtGauge([firstID lastID], sprintf('Phase %02d: ', phaseID), ...
                'verbose', obj.conf.verbose);
            for grainID = firstID : lastID
                gauge.incrementAndDisplay();

                try
                    grain_file = fullfile(phaseDir, sprintf('grain_%04d.mat', grainID));
                    grain      = load(grain_file, 'vol', 'shift', 'id');
                    % Actually threshold now!
                    parameters = obj.param;
                    if isfield(parameters.rec, 'ANDY') && parameters.rec.ANDY
                        % iterative search
                        grain      = obj.thresholdAutoSingleGrainANDY(grain);
                    else
                        % normal way
                        grain      = obj.thresholdAutoSingleGrain(grain);
                    end
                    seg        = grain.seg;
                    segbb      = grain.segbb;
                    threshold  = grain.threshold;
                    Area       = grain.Area;

                    % Check size and update metadata
                    if isempty(seg)
                        fprintf('\n\nWarning! Empty volume in grain %d.\n\n',grainID);
                        gauge.rePrint();
                        seg = 1;
                        segbb = round([(size(grain.vol) -1), (size(grain.vol) +1)] /2);
                    end
                    sample.phases{phaseID}.setThreshold(grainID, threshold);
                    sample.phases{phaseID}.setBoundingBox(grainID, segbb);

                    % Write to file
                    if verLessThan('matlab', '7.14')
                        save(grain_file, '-append', 'seg', 'segbb', 'threshold', 'Area');
                    else
                        matobj = matfile(grain_file, 'Writable', true);
                        matobj.seg = seg;
                        matobj.segbb = segbb;
                        matobj.threshold = threshold;
                        matobj.Area = Area;
                    end
                catch mexc
                    gtPrintException(mexc, ...
                        'Segmenting grain %d in phase %d failed', grainID, phaseID);
                    gauge.rePrint();
                end

            end
            gauge.delete();

            disp('Now writing to sample.mat')
            samplefile  = fullfile(obj.param.acq.dir, '4_grains', 'sample.mat');
            sample.mergeToFile(filetable, samplefile, 'sample');

            if (nargout >= 1)
                varargout{1} = seg;
            end
        end

        function grain = thresholdSingleGrainInFile(obj, phaseID, grainID, threshold, center)
        % GTTHRESHOLD/THRESHOLDSINGLEGRAININFILE

            fprintf('Segmenting Grain %d, from phase %d.. ', grainID, phaseID);
            filePath = fullfile(obj.param.acq.dir, '4_grains', ...
                                sprintf('phase_%02d', phaseID), ...
                                sprintf('grain_%04d.mat', grainID));
            grain = load(filePath);

            if (exist('center', 'var'))
                do_morpho = true;
            else
                center = zeros(1, 3);
                do_morpho = false;
                warning('THRESHOLD:less_capability', ...
                        'Morphological reconstruction is disabled')
            end
            grain = obj.segmentAndDoMorph(grain, threshold, do_morpho, center);

            seg        = grain.seg;
            segbb      = grain.segbb;
            threshold  = grain.threshold;
            Area       = grain.Area;

            if verLessThan('matlab', '7.14')
                save(filePath, '-append', 'seg', 'segbb', 'threshold', 'Area');
            else
                matobj = matfile(filePath, 'Writable', true);
                matobj.seg = seg;
                matobj.segbb = segbb;
                matobj.threshold = threshold;
                matobj.Area = Area;
            end
            fprintf('Done.\n');
        end

        function segvol = thresholdVolume(obj, volume, threshold, center)
        % GTTHRESHOLD/THRESHOLDVOLUME

            if (exist('center', 'var') && all(center > 0))
                do_morpho = true;
            else
                center = zeros(1, 3);
                do_morpho = false;
                warning('THRESHOLD:less_capability', ...
                        'Morphological reconstruction is disabled')
            end

            mess = 'Error while segmenting the volume';
            segvol = obj.segmentVol(volume, threshold, do_morpho, mess, center);
        end
    end

    methods (Access = protected)
        function grain = thresholdAutoSingleGrain(obj, grain)
        % GTTHRESHOLD/THRESHOLDAUTOSINGLEGRAIN Segment 3D grain volume
        %
        % function grain = thresh.thresholdAutoSingleGrain(grain, rec)
        %
        % INPUTS:  grain.vol          graylevel reconstruction of grain volume
        %          rec                contents of parameters.rec structure:
        %             .percentile2    adaptive threshold see below
        %             .do_morph_recon remove not connected areas by morphological
        %                             reconstruction (<logical> {true})
        % OUTPUTS:
        %          grain.thresh Threshold value
        %          grain.seg    Segmented grain volume
        %          grain.Area   Area of grain
        %          grain.segbb  BoundingBox (gt convention) for insertion of seg
        %                       volume into sample volume
        %
        % explanation - we expect a histogram with a peak corresponding to the
        % grain, and a peak a low values corresponding to the background of zeros.
        % We want a threshold value below the peak corresponding to the grain, but
        % above the background peak.

            if (~isfield(grain, 'vol') || isempty(grain.vol))
                gtError('GtThreshold:wrong_argument', ...
                    'Volume to segment is empty!')
            end

            vol = grain.vol;

            % calulate intensity in the central region of the grain
            centerx = round(size(vol, 1) * 0.45) : round(size(vol, 1) * 0.55);
            centery = round(size(vol, 2) * 0.45) : round(size(vol, 2) * 0.55);
            centerz = round(size(vol, 3) * 0.45) : round(size(vol, 3) * 0.55);

            background = gtImgMeanValue(vol(centerx, centery, centerz)) / 10;
            % Small performance boost
            indexes = vol > background;
            % define threshold as percentile/100 x standard dev below the mean,
            % excluding low (10 percentile background) values
            stdvol  = std( vol(indexes) );
            meanvol = mean( vol(indexes) );

            %if (stdvol > meanvol)
            %    threshold_val = obj.param.rec.percentile1/100 * meanvol;
            %else
            %    threshold_val = meanvol - obj.param.rec.percentile2/100 * stdvol;
            %end
            % The above cases are not continuous! Use this instead:
            threshold_val = meanvol - obj.param.rec.percentile2/100 * stdvol;
            
            center(1) = round(size(vol, 1) / 2);
            center(2) = round(size(vol, 2) / 2);
            center(3) = round(size(vol, 3) / 2);
            grain = obj.segmentAndDoMorph( grain, threshold_val, ...
                                            obj.param.rec.do_morph_recon, ...
                                            center);
        end

        function grain = segmentAndDoMorph(obj, grain, threshold, do_morpho, center)
        % GTTHRESHOLD/SEGMENTANDDOMORPH

            message = sprintf(['morphological reconstruction failed for ' ...
                               'grain %d - continue without...'], ...
                              grain.id);
            segvol = obj.segmentVol(grain.vol, threshold, do_morpho, message, ...
                                    center);

            regprop = regionprops(segvol, 'BoundingBox', 'Area');
            if (length(regprop) > 1)
                % let's take the biggest! (usually happens without morpho)
                ind     = find([regprop.Area] == max([regprop.Area]), 1);
                regprop = regprop(ind);
            end

            if isempty(regprop)
                grain.seg   = [];
                grain.Area  = [];
                grain.segbb = [];
            else
                segbb = regprop.BoundingBox([2 1 3 5 4 6]);

                % ceil because Matlab regionprops yields "0.5" (instead of 1)
                % for bb start values
                grain.seg       = gtCrop(segvol, ceil(segbb));
                grain.Area      = regprop.Area;
                segbb(1:3)      = grain.shift + floor(segbb(1:3));
                grain.segbb     = segbb;
            end
            grain.threshold = threshold;
        end

        function segvol = segmentVol(~, vol, threshold, do_morpho, errorMsg, center)
        % GTTHRESHOLD/SEGMENTVOL

            if (isempty(vol))
                gtError('GtThreshold:wrong_argument', ...
                    'Volume to segment is empty!')
            end

            segvol = vol > threshold;

            % morphological reconstruction removes not connected areas
            if (do_morpho && all(center > 0))
                try
                    marker = false(size(vol));
                    marker(center(1), center(2), center(3)) = true;
                    marker = marker & segvol;
                    segvol = double(imreconstruct(marker, segvol));
                catch mexc
                    gtPrintException(mexc, errorMsg);
                end
            end
        end
        
                function grain = thresholdAutoSingleGrainANDY(obj, grain)
        % GTTHRESHOLD/THRESHOLDAUTOSINGLEGRAINANDY Segment 3D grain volume
        %
        %
        % Test version, Andy 
        %
        % function grain = thresh.thresholdAutoSingleGrain(grain, rec)
        %
        % INPUTS:  grain.vol          graylevel reconstruction of grain volume
        %          rec                contents of parameters.rec structure:
        %             .percentile2    adaptive threshold see below
        %             .do_morph_recon remove not connected areas by morphological
        %                             reconstruction (<logical> {true})
        % OUTPUTS:
        %          grain.thresh Threshold value
        %          grain.seg    Segmented grain volume
        %          grain.Area   Area of grain
        %          grain.segbb  BoundingBox (gt convention) for insertion of seg
        %                       volume into sample volume
        %
        % explanation - we want to end up with a grain that has a volume
        % which is consistent with the difspot sizes.  Adjust the threshold
        % until we achieve this.  Probably need another field in
        % parameters.rec.ANDY_f and a constant.

            if (~isfield(grain, 'vol') || isempty(grain.vol))
                gtError('GtThreshold:wrong_argument', ...
                    'Volume to segment is empty!')
            end

            vol = grain.vol;

            % slow and stupid for the moment
            
            % get bounding box sizes from pair table
            [bbx, bby] = mym(sprintf('select avbbXsize, avbbYsize from %s where grainid=%d',...
                obj.param.acq.pair_tablename, grain.id));
            bbx = median(bbx);
            bby = median(bby);
            target = bbx * bbx * bby * obj.param.rec.ANDY_f ;
  
            % calulate intensity in the central region of the grain
            centerx = round(size(vol, 1) * 0.45) : round(size(vol, 1) * 0.55);
            centery = round(size(vol, 2) * 0.45) : round(size(vol, 2) * 0.55);
            centerz = round(size(vol, 3) * 0.45) : round(size(vol, 3) * 0.55);
            % use this as a first guess
            threshold_val = gtImgMeanValue(vol(centerx, centery, centerz)) ;
            % centre point for morphological
            center(1) = round(size(vol, 1) / 2);
            center(2) = round(size(vol, 2) / 2);
            center(3) = round(size(vol, 3) / 2);
       
            happy = false; %:-(
            count = 0;
            while ~happy && count<obj.param.rec.ANDY_nitr
                % segment, test size
                grain = obj.segmentAndDoMorph( grain, threshold_val, ...
                                            obj.param.rec.do_morph_recon, ...
                                            center);
                test = sum(grain.seg(:));
                delta = 100 * (test-target)/target
                
                if abs(delta) > 1
                    
                    delta_sign = sign(delta); 
                    delta_mag = min(threshold_val*abs(delta)/100, 0.2); 
                    
                    threshold_val = threshold_val + (delta_sign * delta_mag);
                else
                    happy = true; %:-)
                end
                count = count+1 ;
            end
                disp(sprintf('finished grain %d after %d iterations, error %0.2f', grain.id, count, delta))

        end 
    end
end
