function [images,u_direct,v_direct,u_diffr,v_diffr,Omega,Eta,Theta] = gtPredictPositions_3D(phaseid, normal, hkl, grainCentroid, plot_flag, parameters, codeflag)
% GTPREDICTPOSITIONS_3D  predict positions of spots in images
%     [images,u_direct,v_direct,u_diffr,v_diffr,Omega,Eta,Theta] = gtPredictPositions_3D(phaseid, normal, hkl, grainCentroid, plot_flag, parameters, codeflag)
%     -----------------------------------------------------------------------------------------------------------------------------------------------
%
%     if parameters.geo includes the neccessary fields for the arbitary
%     geometry, use the FED functions to predict the position of the
%     diffraction spot on the detector.
%     this may be a bit temporary until; FED has been updated
%
%     take a plane normal, its hkl type (to calc Theta), and a grainCentroid [x,y,z], and returns lists of max images, and x and y
%     positions of the ext and dif spots.
%     plot_flag=1 plots images
%
%     Version 005 01-08-2013 by LNervo
%       Replaced gtCalculateDist with gtCrystDSpacing and gtCrystTheta
%
%     Modified on March 08 and March 14, 2012 by PReischig
%       Adapted input for gtCalculateDist.
%    
%     Modified on January 18, 2012 by LNervo
%       Cleaning and formatting
%
%     Modified on June 28, 2011 by LNervo
%       Replace forward simulation part with the FED code
%       Add a flag to use the old code eve if the fed parameters are in
%       Add fed reordering of Omega
%     
%     Modified on March 08, 2009 by Marcelo, Andy, Wolfgang
%       Works with old coordinate system 

acq=parameters.acq;
geo=parameters.geo;%obsolete...it does not exist anymore
labgeo = parameters.labgeo;

% do we have all the required fields?
FEDgeofields{1}='detpos0';
FEDgeofields{2}='detdiru0';
FEDgeofields{3}='detdirv0';
FEDgeofields{4}='beamdir0';
FEDgeofields{5}='rotdir0';
FEDgeofields{6}='detucen0';
FEDgeofields{7}='detvcen0';

% newcode
if all(isfield(geo, FEDgeofields)) && codeflag
    % use the new functions, allowing arbitary geometry
    sam       = gtSampleGeoInSampleSystem(parameters);
    % matrices for computing rotation tensors / arbitary geometry unpacking
    rotcomp   = gtFedRotationMatrixComp(geo.rotdir0');
    beamdir0  = geo.beamdir0';
    % detector geometry
    det.uvcen = [geo.detucen0, geo.detvcen0]';
    det.Q     = gtFedDetectorProjectionTensor(geo.detdiru0', geo.detdirv0',1,1); %det.Q=gtFedDetectorProjectionTensor(geo.detdiru0, geo.detdirv0)
    det.p     = geo.detpos0'/acq.pixelsize; % in pixel
    det.n     = cross(geo.detdiru0, geo.detdirv0)';
    % step size
    omstep    = 180/acq.nproj;

    % given a hkl plane, calculate the interplanar distance.
    Theta = gtCrystTheta( gtCrystDSpacing(hkl, gtCrystHKL2CartesianMatrix(parameters.cryst(phaseid).latticepar) ), parameters.acq.energy);
    sinth = sind(Theta);

    % calc Omega angles
    [Omega,pllab,pls] = gtFedPredictOmega(normal',sinth,beamdir0,geo.rotdir0',rotcomp);
    
    if isempty(Omega)
        images   = NaN;
        u_direct = NaN;
        v_direct = NaN;
        u_diffr  = NaN;
        v_diffr  = NaN;
        Theta    = NaN;
        Eta      = NaN;
        Omega    = NaN;
        return;
    end
    
    if ~isempty(Omega)       
        % initial S rotation tensor
        S1 = gtFedRotationTensor(Omega(1), rotcomp);
        S2 = gtFedRotationTensor(Omega(2), rotcomp);
        S3 = gtFedRotationTensor(Omega(3), rotcomp);
        S4 = gtFedRotationTensor(Omega(4), rotcomp);
        
        % Diffraction vector:
        d1 = gtFedPredictDiffVec(pllab(:,1), sinth, beamdir0);
        d2 = gtFedPredictDiffVec(pllab(:,2), sinth, beamdir0);
        d3 = gtFedPredictDiffVec(pllab(:,3), sinth, beamdir0);
        d4 = gtFedPredictDiffVec(pllab(:,4), sinth, beamdir0);
        
        d1 = d1/norm(d1); % d1'/norm(d1)
        d2 = d2/norm(d2);
        d3 = d3/norm(d3);
        d4 = d4/norm(d4);
            
        % Following the convention of the mathcing output, the omega value
        % smaller than 180deg (spot "a") is used in the pair data.
        % The two Friedel pairs are 1a-1b and 2a-2b.

        if Omega(1) < Omega(3)
            Omega1a  = Omega(1);
            s1a   = S1;
            s1b   = S3;
            Omega1b  = Omega(3);
            pl1a  = pls(:,1)';
            pl1b  = pls(:,3)';
            pllab1a = pllab(:,1)';
            pllab1b = pllab(:,3)';
            d1a   = d1';
            d1b   = d3';
        else
            Omega1a  = Omega(3);
            s1a   = S3;
            s1b   = S1;
            Omega1b  = Omega(1);
            pl1a  = pls(:,3)';
            pl1b  = pls(:,1)';
            pllab1a = pllab(:,3)';
            pllab1b = pllab(:,1)';
            d1a   = d3';
            d1b   = d1';
        end
        
        if Omega(2) < Omega(4)
            Omega2a  = Omega(2);
            Omega2b  = Omega(4);
            s2a   = S2;
            s2b   = S4;
            pl2a  = pls(:,2)';
            pl2b  = pls(:,4)';
            pllab2a = pllab(:,2)';
            pllab2b = pllab(:,4)';
            d2a   = d2';
            d2b   = d4';
        else
            Omega2a  = Omega(4);
            Omega2b  = Omega(2);
            s2a   = S4;
            s2b   = S2;
            pl2a  = pls(:,4)';
            pl2b  = pls(:,2)';
            pllab2a = pllab(:,4)';
            pllab2b = pllab(:,2)';
            d2a   = d4';
            d2b   = d2';
        end
        
        Omega=[Omega1a;Omega1b;Omega2a;Omega2b];
        Theta=Theta*ones(size(Omega));
    
        % Eta angles
        eta1a = gtGeoEtaFromDiffVec(d1a, labgeo); % takes row vector
        eta1b = gtGeoEtaFromDiffVec(d1b, labgeo);
        eta2a = gtGeoEtaFromDiffVec(d2a, labgeo);
        eta2b = gtGeoEtaFromDiffVec(d2b, labgeo);

        Eta=[eta1a;eta1b;eta2a;eta2b];

        % u,v coordinates on the detector
        uv1a=gtFedPredictUVW(s1a,d1a',grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega1a,omstep);
        uv1b=gtFedPredictUVW(s1b,d1b',grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega1b,omstep);
        uv2a=gtFedPredictUVW(s2a,d2a',grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega2a,omstep);
        uv2b=gtFedPredictUVW(s2b,d2b',grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega2b,omstep);

        images=round([uv1a(3);uv1b(3);uv2a(3);uv2b(3)]); %added round
        u_diffr=round([uv1a(1);uv1b(1);uv2a(1);uv2b(1)]);
        v_diffr=round([uv1a(2);uv1b(2);uv2a(2);uv2b(2)]);

        % calc u_dir and v_dir
        uv1a_direct=gtFedPredictUVW(s1a,beamdir0,grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega1a,omstep);
        uv1b_direct=gtFedPredictUVW(s1b,beamdir0,grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega1b,omstep);
        uv2a_direct=gtFedPredictUVW(s2a,beamdir0,grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega2a,omstep);
        uv2b_direct=gtFedPredictUVW(s2b,beamdir0,grainCentroid',det.p,det.n,det.Q,det.uvcen,Omega2b,omstep);

        u_direct=round([uv1a_direct(1);uv1b_direct(1);uv2a_direct(1);uv2b_direct(1)]);
        v_direct=round([uv1a_direct(2);uv1b_direct(2);uv2a_direct(2);uv2b_direct(2)]);
    end % end if Omega

else % end arbitrary geometry
    
    % use the standard geometry
    grainx = grainCentroid(1);
    grainy = grainCentroid(2);
    grainz = grainCentroid(3);
    
    % use gtPredictAngles to get Theta, Eta, Omega - gives a pair of angles
    [Theta,Eta,Omega] = gtPredictAngles3D(normal, hkl, parameters);
    
    if ~isempty(Theta)
        %get only Omegas within 0-180 degrees for 180deg data
        if strcmpi(parameters.acq.type,'180degree')
            %disp('180 degree data')
            dummy=find(Omega<0 | Omega>180);
            Theta(dummy)=[];
            Eta(dummy)=[];
            Omega(dummy)=[];
        end
        
        for i=1:length(Omega)
            % lab:  grain position in Labsystem
            [labx,laby,labz]=gtTrSamToLab(grainx,grainy,grainz,Omega(i));
            lab=[labx; laby; labz];
            %diffracted beam direction (normalized)
            d=[cosd(2*Theta(i));-sind(2*Theta(i)).*sind(Eta(i));sind(2*Theta(i)).*cosd(Eta(i))];
            
            % intersection of diffraction beam with detector plane:  labx+f*dx=detx
            f=(acq.dist/acq.pixelsize+parameters.match.corr_rot_to_det-labx)/d(1);
            % diffraction spot position
            pos=lab+f*d;
            
            u_diffr(i)=acq.rotu+pos(2);       % old convention
            v_diffr(i)=acq.ydet/2-pos(3);
            
            u_direct(i)=acq.rotu+lab(2);      % old convention
            v_direct(i)=acq.ydet/2-lab(3);
        end
        
        %round to integer values
        u_direct = round(u_direct);
        v_direct = round(v_direct);
        u_diffr  = round(u_diffr);
        v_diffr  = round(v_diffr);
        %get number of full image
        images   = round((Omega/180)*acq.nproj);

    else
        images   = NaN;
        u_direct = NaN;
        v_direct = NaN;
        u_diffr  = NaN;
        v_diffr  = NaN;
        Theta    = NaN;
        Eta      = NaN;
        Omega    = NaN;
    end
    
end % end standard geometry

%~~~~~~~~plot results on full images~~~~~~~~~~~~~~~~~~~~~~
if plot_flag
    
    for i=1:size(Omega,2)
        
        %convert Omega to image number
        start_image = images(i)-5;
        if start_image<0
            start_image=0;
        end
        end_image = images(i)+5;
        if end_image>(2*acq.nproj)-1;
            end_image=(2*acq.nproj)-1;
        end
        full=zeros(acq.ydet,acq.xdet);
        for im=start_image:end_image
            full=full+edf_read( fullfile('1_preprocessing','full', sprintf('full%04d.edf',im)) );
        end
        full=full./(end_image-start_image+1);
        
        % ajout sabine%
        close all

        figure();
        imshow((full),[-10 10]);
        text=sprintf('%d %d %d reflection',hkl(1),hkl(2), hkl(3));
        title(text);
        hold on;
        plot(u_direct(i),v_direct(i),'go');
        plot(u_diffr(i),v_diffr(i),'ro');

        disp('click image to close and see next spot (if exists...)')
        k=waitforbuttonpress;
    end
end
%~~~~~~~~plot results on full images~~~~~~~~~~~~~~~~~~~~~~

end %end of function
