%
% function theta=gtThetaOfPairs(scXA,scYA,scXB,scYB,rottodet,tiltY,tiltZ,rotx,sorZ)
%
% Given the Scintillator coordinates of a pair of diffraction spots, 
% computes the real theta angle they correspond to, according to tilts and 
% rotation axis to detector (scintillator) distance.
%
%   LAB system is right-handed and defined as: 
%     origin X = on the rotation axis, positive towards the camera
%     origin Y = on the rotation axis
%     origin Z = floor(parameters.acq.ydet/2) in the middle of the central
%                pixel row of the image, positive upwards
% 
% INPUT  scXA = coloumn vector of X coordinates on the Scintillator of point A (pixels or mm-s)
%        scYA = coloumn vector of Y coordinates on the Scintillator of point A (pixels or mm-s)
%        scXB = coloumn vector of X coordinates on the Scintillator of point B (pixels or mm-s)
%        scYB = coloumn vector of Y coordinates on the Scintillator of point B (pixels or mm-s)
%        rottodet = rotation axis to detector (scintillator) distance at rotx,sorZ (pixels or mm-s)
%        /scX,scY,rottodet have to be in the same units; e.g. pixels or mm-s/
%        tiltY = tilt around axis Y in Lab system (in degrees)
%        tiltZ = tilt around axis Z in Lab system (in degrees)
%        rotx = location of rotation axis in the images (X coord. in pixels)
%        sorZ = Z=0 location in the images (Y coord. in pixels at the rotation axis) 
%
%        e.g. ([234;334;434],[1444;1644;1944],[134;234;534],[555;545;535],
%              3600,0.123,0.421,1026,1024)
%
% OUTPUT theta = coloumn vector of real theta angles
%
%        e.g. [7.3948; 7.1822; 6.9281]
%
%

function theta=gtThetaOfPairs(scXA,scYA,scXB,scYB,rottodet,tiltY,tiltZ,rotx,sorZ)

% Dv1=2*rottodet-sind(tiltZ).*(scXA+scXB-2*rotx)-sind(tiltY).*cosd(tiltZ).*(scYA+scYB-2*sorZ) ;
% Dv2=cosd(tiltZ).*(scXA+scXB-2*rotx)-sind(tiltY).*sind(tiltZ).*(scYA+scYB-2*sorZ) ;
% Dv3=-cosd(tiltY).*(scYA-scYB) ;
% 
% theta=0.5*acosd(Dv1./sqrt(Dv1.^2+Dv2.^2+Dv3.^2)) ;

Dv=gtDiffVecInLab(scXA,scYA,scXB,scYB,rottodet,tiltY,tiltZ,rotx,sorZ);

theta=0.5*acosd(Dv(:,1)./sqrt(Dv(:,1).^2+Dv(:,2).^2+Dv(:,3).^2));

end
